<html>
    <head>
        <title>Django-SES Example</title>
        <style type="text/css">
            font-family: Helvetica, freesans, Arial, sans-serif;
        </style>
    </head>
    <body>
        <h1>Django-SES Example</h1>
        {% block content %}{% endblock %}
    </body>
</html>
